"""Python wrappers around TensorFlow ops.

This file is MACHINE GENERATED! Do not edit.
Original C++ source file: mpi_ops.cc
"""

import collections as _collections
import six as _six

from tensorflow.python import pywrap_tensorflow as _pywrap_tensorflow
from tensorflow.python.eager import context as _context
from tensorflow.python.eager import core as _core
from tensorflow.python.eager import execute as _execute
from tensorflow.python.framework import dtypes as _dtypes
from tensorflow.python.framework import errors as _errors
from tensorflow.python.framework import tensor_shape as _tensor_shape

from tensorflow.core.framework import op_def_pb2 as _op_def_pb2
# Needed to trigger the call to _set_call_cpp_shape_fn.
from tensorflow.python.framework import common_shapes as _common_shapes
from tensorflow.python.framework import op_def_registry as _op_def_registry
from tensorflow.python.framework import ops as _ops
from tensorflow.python.framework import op_def_library as _op_def_library
from tensorflow.python.util.tf_export import tf_export


@tf_export('mpi_allgather')
def mpi_allgather(tensor, sizes, name=None):
  r"""Perform an MPI Allgather on a tensor. All other processes that do a gather on a

  tensor with the same name must have the same rank for that tensor, and have the
  same dimension on all but the first dimension.

  Arguments
      tensor:     A tensor to gather.
      sizes:      A tensor containing the first-dimension sizes of tensors to be
                  gathered from other ranks

  Output
      gathered:   A tensor with the same shape as `tensor` except for the first
                  dimension, which is the sum of dimensions in `sizes`.

  Args:
    tensor: A `Tensor`. Must be one of the following types: `int32`, `int64`, `float32`.
    sizes: A `Tensor`. Must be one of the following types: `int64`.
    name: A name for the operation (optional).

  Returns:
    A `Tensor`. Has the same type as `tensor`.
  """
  _ctx = _context.context()
  if not _ctx.executing_eagerly():
    _, _, _op = _op_def_lib._apply_op_helper(
        "MPIAllgather", tensor=tensor, sizes=sizes, name=name)
    _result = _op.outputs[:]
    _inputs_flat = _op.inputs
    _attrs = ("T", _op.get_attr("T"), "S", _op.get_attr("S"))
    _execute.record_gradient(
      "MPIAllgather", _inputs_flat, _attrs, _result, name)
    _result, = _result
    return _result

  else:
    try:
      _result = _pywrap_tensorflow.TFE_Py_FastPathExecute(
        _ctx._handle, _ctx.device_name, "MPIAllgather", name,
        _ctx._post_execution_callbacks, tensor, sizes)
      return _result
    except _core._FallbackException:
      return mpi_allgather_eager_fallback(
          tensor, sizes, name=name)
    except _core._NotOkStatusException as e:
      if name is not None:
        message = e.message + " name: " + name
      else:
        message = e.message
      _six.raise_from(_core._status_to_exception(e.code, message), None)


def mpi_allgather_eager_fallback(tensor, sizes, name=None):
  r"""This is the slowpath function for Eager mode.
  This is for function mpi_allgather
  """
  _ctx = _context.context()
  _attr_T, (tensor,) = _execute.args_to_matching_eager([tensor], _ctx)
  _attr_S, (sizes,) = _execute.args_to_matching_eager([sizes], _ctx)
  _inputs_flat = [tensor, sizes]
  _attrs = ("T", _attr_T, "S", _attr_S)
  _result = _execute.execute(b"MPIAllgather", 1, inputs=_inputs_flat,
                             attrs=_attrs, ctx=_ctx, name=name)
  _execute.record_gradient(
      "MPIAllgather", _inputs_flat, _attrs, _result, name)
  _result, = _result
  return _result

_ops.RegisterShape("MPIAllgather")(None)


@tf_export('mpi_allreduce')
def mpi_allreduce(tensor, name=None):
  r"""Perform an MPI Allreduce on a tensor. All other processes that do a reduction

  on a tensor with the same name must have the same dimension for that tensor.
  Tensors are reduced with other tensors that have the same node name for the
  allreduce.

  Arguments
      tensor:     A tensor to reduce.

  Output
      sum:        A tensor with the same shape as `tensor`, summed across all
                  MPI processes.

  Args:
    tensor: A `Tensor`. Must be one of the following types: `int32`, `int64`, `float32`.
    name: A name for the operation (optional).

  Returns:
    A `Tensor`. Has the same type as `tensor`.
  """
  _ctx = _context.context()
  if not _ctx.executing_eagerly():
    _, _, _op = _op_def_lib._apply_op_helper(
        "MPIAllreduce", tensor=tensor, name=name)
    _result = _op.outputs[:]
    _inputs_flat = _op.inputs
    _attrs = ("T", _op.get_attr("T"))
    _execute.record_gradient(
      "MPIAllreduce", _inputs_flat, _attrs, _result, name)
    _result, = _result
    return _result

  else:
    try:
      _result = _pywrap_tensorflow.TFE_Py_FastPathExecute(
        _ctx._handle, _ctx.device_name, "MPIAllreduce", name,
        _ctx._post_execution_callbacks, tensor)
      return _result
    except _core._FallbackException:
      return mpi_allreduce_eager_fallback(
          tensor, name=name)
    except _core._NotOkStatusException as e:
      if name is not None:
        message = e.message + " name: " + name
      else:
        message = e.message
      _six.raise_from(_core._status_to_exception(e.code, message), None)


def mpi_allreduce_eager_fallback(tensor, name=None):
  r"""This is the slowpath function for Eager mode.
  This is for function mpi_allreduce
  """
  _ctx = _context.context()
  _attr_T, (tensor,) = _execute.args_to_matching_eager([tensor], _ctx)
  _inputs_flat = [tensor]
  _attrs = ("T", _attr_T)
  _result = _execute.execute(b"MPIAllreduce", 1, inputs=_inputs_flat,
                             attrs=_attrs, ctx=_ctx, name=name)
  _execute.record_gradient(
      "MPIAllreduce", _inputs_flat, _attrs, _result, name)
  _result, = _result
  return _result

_ops.RegisterShape("MPIAllreduce")(None)


@tf_export('mpi_init')
def mpi_init(name=None):
  r"""Initialize MPI for the current process.

  If this is run on a GPU, then that GPU must be used for all future MPI
  operations. If it is run on CPU, then all future MPI operations must also
  run on CPU.

  Args:
    name: A name for the operation (optional).

  Returns:
    The created Operation.
  """
  _ctx = _context.context()
  if not _ctx.executing_eagerly():
    _, _, _op = _op_def_lib._apply_op_helper(
        "MPIInit", name=name)
    return _op
    _result = None
    return _result

  else:
    try:
      _result = _pywrap_tensorflow.TFE_Py_FastPathExecute(
        _ctx._handle, _ctx.device_name, "MPIInit", name,
        _ctx._post_execution_callbacks)
      return _result
    except _core._FallbackException:
      return mpi_init_eager_fallback(
          name=name)
    except _core._NotOkStatusException as e:
      if name is not None:
        message = e.message + " name: " + name
      else:
        message = e.message
      _six.raise_from(_core._status_to_exception(e.code, message), None)


def mpi_init_eager_fallback(name=None):
  r"""This is the slowpath function for Eager mode.
  This is for function mpi_init
  """
  _ctx = _context.context()
  _inputs_flat = []
  _attrs = None
  _result = _execute.execute(b"MPIInit", 0, inputs=_inputs_flat, attrs=_attrs,
                             ctx=_ctx, name=name)
  _result = None
  return _result

_ops.RegisterShape("MPIInit")(None)


@tf_export('mpi_local_rank')
def mpi_local_rank(name=None):
  r"""Returns the index of the current process in the node it is on.

  More precisely, returns the rank of the calling process in communicator that
  only spans the MPI processes running on that node.

  Args:
    name: A name for the operation (optional).

  Returns:
    A `Tensor` of type `int32`.
    Rank of the calling process on the node it is on.
  """
  _ctx = _context.context()
  if not _ctx.executing_eagerly():
    _, _, _op = _op_def_lib._apply_op_helper(
        "MPILocalRank", name=name)
    _result = _op.outputs[:]
    _inputs_flat = _op.inputs
    _attrs = None
    _execute.record_gradient(
      "MPILocalRank", _inputs_flat, _attrs, _result, name)
    _result, = _result
    return _result

  else:
    try:
      _result = _pywrap_tensorflow.TFE_Py_FastPathExecute(
        _ctx._handle, _ctx.device_name, "MPILocalRank", name,
        _ctx._post_execution_callbacks)
      return _result
    except _core._FallbackException:
      return mpi_local_rank_eager_fallback(
          name=name)
    except _core._NotOkStatusException as e:
      if name is not None:
        message = e.message + " name: " + name
      else:
        message = e.message
      _six.raise_from(_core._status_to_exception(e.code, message), None)


def mpi_local_rank_eager_fallback(name=None):
  r"""This is the slowpath function for Eager mode.
  This is for function mpi_local_rank
  """
  _ctx = _context.context()
  _inputs_flat = []
  _attrs = None
  _result = _execute.execute(b"MPILocalRank", 1, inputs=_inputs_flat,
                             attrs=_attrs, ctx=_ctx, name=name)
  _execute.record_gradient(
      "MPILocalRank", _inputs_flat, _attrs, _result, name)
  _result, = _result
  return _result

_ops.RegisterShape("MPILocalRank")(None)


@tf_export('mpi_rank')
def mpi_rank(name=None):
  r"""Returns the index of the current process in the MPI group.

  More precisely, returns the rank of the calling process in the MPI_COMM_WORLD
  communicator.

  Args:
    name: A name for the operation (optional).

  Returns:
    A `Tensor` of type `int32`. Rank of the calling process.
  """
  _ctx = _context.context()
  if not _ctx.executing_eagerly():
    _, _, _op = _op_def_lib._apply_op_helper(
        "MPIRank", name=name)
    _result = _op.outputs[:]
    _inputs_flat = _op.inputs
    _attrs = None
    _execute.record_gradient(
      "MPIRank", _inputs_flat, _attrs, _result, name)
    _result, = _result
    return _result

  else:
    try:
      _result = _pywrap_tensorflow.TFE_Py_FastPathExecute(
        _ctx._handle, _ctx.device_name, "MPIRank", name,
        _ctx._post_execution_callbacks)
      return _result
    except _core._FallbackException:
      return mpi_rank_eager_fallback(
          name=name)
    except _core._NotOkStatusException as e:
      if name is not None:
        message = e.message + " name: " + name
      else:
        message = e.message
      _six.raise_from(_core._status_to_exception(e.code, message), None)


def mpi_rank_eager_fallback(name=None):
  r"""This is the slowpath function for Eager mode.
  This is for function mpi_rank
  """
  _ctx = _context.context()
  _inputs_flat = []
  _attrs = None
  _result = _execute.execute(b"MPIRank", 1, inputs=_inputs_flat, attrs=_attrs,
                             ctx=_ctx, name=name)
  _execute.record_gradient(
      "MPIRank", _inputs_flat, _attrs, _result, name)
  _result, = _result
  return _result

_ops.RegisterShape("MPIRank")(None)


@tf_export('mpi_size')
def mpi_size(name=None):
  r"""Returns the number of running MPI processes.

  More precisely, returns the number of MPI processes in the group associated
  with the MPI_COMM_WORLD communicator.

  Args:
    name: A name for the operation (optional).

  Returns:
    A `Tensor` of type `int32`. Size of the MPI group.
  """
  _ctx = _context.context()
  if not _ctx.executing_eagerly():
    _, _, _op = _op_def_lib._apply_op_helper(
        "MPISize", name=name)
    _result = _op.outputs[:]
    _inputs_flat = _op.inputs
    _attrs = None
    _execute.record_gradient(
      "MPISize", _inputs_flat, _attrs, _result, name)
    _result, = _result
    return _result

  else:
    try:
      _result = _pywrap_tensorflow.TFE_Py_FastPathExecute(
        _ctx._handle, _ctx.device_name, "MPISize", name,
        _ctx._post_execution_callbacks)
      return _result
    except _core._FallbackException:
      return mpi_size_eager_fallback(
          name=name)
    except _core._NotOkStatusException as e:
      if name is not None:
        message = e.message + " name: " + name
      else:
        message = e.message
      _six.raise_from(_core._status_to_exception(e.code, message), None)


def mpi_size_eager_fallback(name=None):
  r"""This is the slowpath function for Eager mode.
  This is for function mpi_size
  """
  _ctx = _context.context()
  _inputs_flat = []
  _attrs = None
  _result = _execute.execute(b"MPISize", 1, inputs=_inputs_flat, attrs=_attrs,
                             ctx=_ctx, name=name)
  _execute.record_gradient(
      "MPISize", _inputs_flat, _attrs, _result, name)
  _result, = _result
  return _result

_ops.RegisterShape("MPISize")(None)

def _InitOpDefLibrary(op_list_proto_bytes):
  op_list = _op_def_pb2.OpList()
  op_list.ParseFromString(op_list_proto_bytes)
  _op_def_registry.register_op_list(op_list)
  op_def_lib = _op_def_library.OpDefLibrary()
  op_def_lib.add_op_list(op_list)
  return op_def_lib
# op {
#   name: "MPIAllgather"
#   input_arg {
#     name: "tensor"
#     type_attr: "T"
#   }
#   input_arg {
#     name: "sizes"
#     type_attr: "S"
#   }
#   output_arg {
#     name: "gathered"
#     type_attr: "T"
#   }
#   attr {
#     name: "T"
#     type: "type"
#     allowed_values {
#       list {
#         type: DT_INT32
#         type: DT_INT64
#         type: DT_FLOAT
#       }
#     }
#   }
#   attr {
#     name: "S"
#     type: "type"
#     allowed_values {
#       list {
#         type: DT_INT64
#       }
#     }
#   }
# }
# op {
#   name: "MPIAllreduce"
#   input_arg {
#     name: "tensor"
#     type_attr: "T"
#   }
#   output_arg {
#     name: "sum"
#     type_attr: "T"
#   }
#   attr {
#     name: "T"
#     type: "type"
#     allowed_values {
#       list {
#         type: DT_INT32
#         type: DT_INT64
#         type: DT_FLOAT
#       }
#     }
#   }
# }
# op {
#   name: "MPIInit"
# }
# op {
#   name: "MPILocalRank"
#   output_arg {
#     name: "rank"
#     type: DT_INT32
#   }
# }
# op {
#   name: "MPIRank"
#   output_arg {
#     name: "rank"
#     type: DT_INT32
#   }
# }
# op {
#   name: "MPISize"
#   output_arg {
#     name: "size"
#     type: DT_INT32
#   }
# }
_op_def_lib = _InitOpDefLibrary(b"\n\\\n\014MPIAllgather\022\013\n\006tensor\"\001T\022\n\n\005sizes\"\001S\032\r\n\010gathered\"\001T\"\022\n\001T\022\004type:\007\n\0052\003\003\t\001\"\020\n\001S\022\004type:\005\n\0032\001\t\n9\n\014MPIAllreduce\022\013\n\006tensor\"\001T\032\010\n\003sum\"\001T\"\022\n\001T\022\004type:\007\n\0052\003\003\t\001\n\t\n\007MPIInit\n\030\n\014MPILocalRank\032\010\n\004rank\030\003\n\023\n\007MPIRank\032\010\n\004rank\030\003\n\023\n\007MPISize\032\010\n\004size\030\003")
